#include <setjmp.h>
#include "bsp.h"
#include "fatfs.h"
#include "tty.h"

#define RAM_START 0x2400
#define RAM_END   0x6000

typedef struct {
  void (*main_a)();
  void (*cprintf_a)(const char *,...);
  void (*dllexit_a)();
  int (*tty_ready_a)();
  int (*tty_readc_a)();
  int (*wait_ms_a)();
  int (*tty_rawc_a)();
  int (*random_a)();
  int (*srandom_a)();
} DLL_S;

#define dll (*(DLL_S *)RAM_START)

static jmp_buf dll_buf;

static void
dll_exit ()
{
  p10.b = 0xf0;
  longjmp (dll_buf, 1);
}

static unsigned long next = 1;

static int
random()
{
  next = next * 846793005L + 1;
  return (int)(next >> 3);
}

static void
srandom (unsigned long long n)
{
  next = n;
}

void
dll_load ()
{
  char *ramptr = (char *)RAM_START;
  char *endptr = (char *)RAM_END;
  int r;

  if (!open_file ("default.dll"))
    {
      cprintf("no default.dll found\n");
      return;
    }

  while ((r = read_file (ramptr, endptr - ramptr)) > 0)
    ramptr += r;

  cprintf("%d bytes read\n", ramptr - (char *)RAM_START);

  dll.cprintf_a = &cprintf;
  dll.dllexit_a = &dll_exit;
  dll.tty_ready_a = &tty_ready;
  dll.tty_readc_a = &tty_readc;
  dll.wait_ms_a = &wait_ms;
  dll.tty_rawc_a = &tty_rawc;
  dll.random_a = &random;
  dll.srandom_a = &srandom;

  cprintf("about to call the dll\n");
  if (setjmp (dll_buf) == 0)
    dll.main_a();
  cprintf("back from the dll\n");

  while (tty_ready ())
    tty_readc ();
}
